@@ -14,7 +14,8 @@ fn main() {
 
     println!("cargo:root={}", path_to_str(&out_dir));
 
-    if !target.contains("-linux-") || target.ends_with("-kernel") {
+    let target_is_linux = target.contains("-linux-") || target.ends_with("-linux");
+    if !target_is_linux || target.ends_with("-kernel") {
         return; // Nothing to build for this architecture.
     }
 
@@ -76,19 +77,11 @@ impl CompilerSearchPaths {
     fn new(include_dir: Option<PathBuf>, link_dir: Option<PathBuf>) -> Self {
         env::set_var("LANG", "C");
 
-        let include_paths = if let Some(include_dir) = include_dir {
-            vec![include_dir]
-        } else {
-            Self::get_compiler_include_paths()
-                .expect("selinux-sys: Failed to discover default compiler search paths")
-        };
+        let include_paths = Self::get_compiler_include_paths(include_dir)
+            .expect("selinux-sys: Failed to discover default compiler search paths");
 
-        let link_paths = if let Some(link_dir) = link_dir {
-            vec![link_dir]
-        } else {
-            Self::get_compiler_link_paths()
-                .expect("selinux-sys: Failed to discover default linker search paths")
-        };
+        let link_paths = Self::get_compiler_link_paths(link_dir)
+            .expect("selinux-sys: Failed to discover default linker search paths");
 
         CompilerSearchPaths {
             include_paths,
@@ -96,17 +89,21 @@ impl CompilerSearchPaths {
         }
     }
 
-    fn get_compiler_include_paths() -> io::Result<Vec<PathBuf>> {
-        let compiler = cc::Build::new()
+    fn get_compiler_include_paths(include_dir: Option<PathBuf>) -> io::Result<Vec<PathBuf>> {
+        let mut compiler_builder = cc::Build::new();
+
+        if let Some(include_dir) = include_dir.as_deref() {
+            compiler_builder.include(include_dir);
+        }
+
+        let child = compiler_builder
             .flag("-E")
             .flag("-v")
             .flag("-x")
             .flag("c")
-            .get_compiler();
-
-        let child = compiler
+            .get_compiler()
             .to_command()
-            .arg("-")
+            .arg("-") // stdin
             .stdin(process::Stdio::null())
             .stdout(process::Stdio::null())
             .stderr(process::Stdio::piped())
@@ -122,27 +119,38 @@ impl CompilerSearchPaths {
             ));
         }
 
-        let mut paths: Vec<PathBuf> = output
-            .stderr
-            .split(|&b| b == b'\n')
-            .skip_while(|&line| line != b"#include <...> search starts here:")
-            .take_while(|&line| line != b"End of search list.")
-            .filter_map(|bytes| str::from_utf8(bytes).ok())
-            .map(str::trim)
-            .filter_map(|s| dunce::canonicalize(s).ok())
-            .collect();
+        let mut paths = Vec::with_capacity(8);
+
+        if let Some(include_dir) = include_dir {
+            paths.push(include_dir);
+        }
+
+        paths.extend(
+            output
+                .stderr
+                .split(|&b| b == b'\n')
+                .skip_while(|&line| line != b"#include <...> search starts here:")
+                .take_while(|&line| line != b"End of search list.")
+                .filter_map(|bytes| str::from_utf8(bytes).ok())
+                .map(str::trim)
+                .filter_map(|s| dunce::canonicalize(s).ok()),
+        );
 
         paths.dedup();
         Ok(paths)
     }
 
-    fn get_compiler_link_paths() -> io::Result<Vec<PathBuf>> {
-        let compiler = cc::Build::new()
+    fn get_compiler_link_paths(link_dir: Option<PathBuf>) -> io::Result<Vec<PathBuf>> {
+        let mut compiler_builder = cc::Build::new();
+
+        if let Some(link_dir) = link_dir.as_deref() {
+            compiler_builder.flag("-L").flag(path_to_str(link_dir));
+        }
+
+        let child = compiler_builder
             .flag("-v")
             .flag("-print-search-dirs")
-            .get_compiler();
-
-        let child = compiler
+            .get_compiler()
             .to_command()
             .stdout(process::Stdio::piped())
             .stderr(process::Stdio::null())
@@ -161,7 +169,7 @@ impl CompilerSearchPaths {
         let line = output
             .stdout
             .split(|&b| b == b'\n')
-            .find_map(|line| line.strip_prefix(b"libraries:"))
+            .find_map(|line| strip_bytes_prefix(line, b"libraries:"))
             .and_then(|bytes| str::from_utf8(bytes).ok())
             .map(str::trim)
             .map(|line| line.trim_start_matches('='))
@@ -172,7 +180,11 @@ impl CompilerSearchPaths {
                 )
             })?;
 
-        let mut paths = Vec::<PathBuf>::with_capacity(8);
+        let mut paths = Vec::with_capacity(8);
+
+        if let Some(link_dir) = link_dir {
+            paths.push(link_dir);
+        }
 
         if let Some(lib_paths) = env::var_os("LIBRARY_PATH") {
             paths.extend(env::split_paths(&lib_paths).filter_map(|s| dunce::canonicalize(s).ok()));
@@ -190,19 +202,19 @@ fn target_env_var_os(name: &str, target: &str) -> Option<OsString> {
 
     let target_underscores = target.replace('-', "_");
 
-    env::var_os(format!("{}_{}", name, target))
-        .or_else(|| env::var_os(format!("{}_{}", name, target_underscores)))
-        .or_else(|| env::var_os(format!("TARGET_{}", name)))
-        .or_else(|| env::var_os(name.to_string()))
+    env::var_os(format!("{name}_{target}"))
+        .or_else(|| env::var_os(format!("{name}_{target_underscores}")))
+        .or_else(|| env::var_os(format!("TARGET_{name}")))
+        .or_else(|| env::var_os(name))
 }
 
 fn rerun_if_env_changed(name: &str, target: &str) {
     let target_underscores = target.replace('-', "_");
 
-    println!("cargo:rerun-if-env-changed={}_{}", name, target);
-    println!("cargo:rerun-if-env-changed={}_{}", name, target_underscores);
-    println!("cargo:rerun-if-env-changed=TARGET_{}", name);
-    println!("cargo:rerun-if-env-changed={}", name);
+    println!("cargo:rerun-if-env-changed={name}_{target}");
+    println!("cargo:rerun-if-env-changed={name}_{target_underscores}");
+    println!("cargo:rerun-if-env-changed=TARGET_{name}");
+    println!("cargo:rerun-if-env-changed={name}");
 }
 
 fn rerun_if_dir_changed(dir: &Path) {
@@ -220,7 +232,7 @@ fn rerun_if_dir_changed(dir: &Path) {
 
 fn find_and_output_include_dir(include_paths: &[PathBuf]) -> PathBuf {
     let include_path = find_file_in_dirs("selinux/selinux.h", include_paths)
-        .expect("selinux-sys: Failed to find 'selinux/selinux.h'");
+        .expect("selinux-sys: Failed to find 'selinux/selinux.h'. Please make sure the C header files of libselinux are installed and accessible");
 
     rerun_if_dir_changed(&include_path.join("selinux"));
 
@@ -265,18 +277,14 @@ fn find_and_output_lib_dir(link_paths: &[PathBuf], target: &str, explicit_static
             return;
         }
 
-        if let Some(link_path) = link_paths.get(0) {
+        if let Some(link_path) = link_paths.first() {
             let triplet = target.replace("-unknown-", "-").replace("-none-", "-");
 
-            for &lib_dir in &[
-                link_path,
-                &link_path.join(&target),
-                &link_path.join(&triplet),
-            ] {
+            for &lib_dir in &[link_path, &link_path.join(target), &link_path.join(triplet)] {
                 let lib_path = lib_dir.join(&file_name);
                 if let Ok(md) = lib_path.metadata() {
                     if md.is_file() {
-                        output_lib_dir(&lib_dir, &lib_path, static_lib);
+                        output_lib_dir(lib_dir, &lib_path, static_lib);
                         return;
                     }
                 }
@@ -287,7 +295,7 @@ fn find_and_output_lib_dir(link_paths: &[PathBuf], target: &str, explicit_static
 
 fn generate_bindings(out_dir: &Path, include_path: &Path) {
     let mut builder = bindgen::Builder::default()
-        .rustfmt_bindings(true)
+        .parse_callbacks(Box::new(bindgen::CargoCallbacks::new()))
         .default_enum_style(bindgen::EnumVariation::ModuleConsts)
         .default_macro_constant_type(bindgen::MacroTypeVariation::Signed)
         .size_t_is_usize(true)
@@ -295,7 +303,8 @@ fn generate_bindings(out_dir: &Path, include_path: &Path) {
         .derive_copy(true)
         .derive_eq(true)
         .derive_ord(true)
-        .impl_debug(true);
+        .impl_debug(true)
+        .clang_args(&["-I", path_to_str(include_path)]);
 
     // Do not expose deprecated functions.
     for &blacklisted_function in &[
@@ -370,3 +379,18 @@ fn find_file_in_dirs(path_suffix: &str, dirs: &[PathBuf]) -> io::Result<PathBuf>
 
     Err(io::ErrorKind::NotFound.into())
 }
+
+// This is used instead of `std::slice::strip_prefix()`, in order to reduce the
+// minimum supported Rust version for this crate.
+// This should be removed once we can assume at least Rust version 1.51.0.
+#[must_use]
+fn strip_bytes_prefix<'a>(slice: &'a [u8], prefix: &'_ [u8]) -> Option<&'a [u8]> {
+    let prefix_len = prefix.len();
+    if prefix_len <= slice.len() {
+        let (head, tail) = slice.split_at(prefix_len);
+        if head == prefix {
+            return Some(tail);
+        }
+    }
+    None
+}
